close all; clear all;

%% VOLBA OBMEDZENEJ ALEBO VOLENJ DIFUZIE
% % priestoru pre obmedzenu difuziu
% xh = 0.005;                 % [mm]      
% yh = 0.005;
% zh = 0.005;

% Priestor pre volnu difuziu
xh = 0.05;                 % [mm]      
yh = 0.05;
zh = 0.05;

%% zakladne parametre
%castic
Nmol = 100000;             % pocet castic
MolSurad = zeros(3,Nmol);   % vsetky castice zacinaju z rovnakeho bodu
%difuzie
dt = 0.0005;                % [s] casovy usek 

D = 0.001;                  % [mm^2/s] difuzna konstanta          
Dkonst = sqrt(2*D*dt);      % [mm] smerodatna odchylka dostribucie pohybu castic za cas-dt

D_x = 0.001;                % difuzne koeficienty v smere x,y,z
D_y = 0.001;       
D_z = 0.001;
Dkonst_x = sqrt(2*D_x*dt);
Dkonst_y = sqrt(2*D_y*dt);
Dkonst_z = sqrt(2*D_z*dt);


%casu
Ncas = 100;                 % pocet casovych usekov      
%gradientu 
g = 2;                      % vysledok sucinu gamma*G*tau, 2-200 pre gradient o sile 47-4700mT/m
Ng = 40;                   % pocet gradientov
G = (0:Ng-1).*g;            
ts = (1:Ncas).*dt;

% vytvorenie prazdnych premennych
STG = zeros(Ncas,Ng);
BTG = zeros(Ncas,Ng);


%% spracovanie
for t=1:Ncas
    %MolSurad = MolSurad + Dkonst*randn(3,Nmol);     % pricitanie nahodnych cisel z normalneho rozlozenia, smerodatna odchylka - Dkonst
    
    nah = randn(3,Nmol);
    MolSurad(1,:) = MolSurad(1,:) + Dkonst_x.*nah(1,:);
    MolSurad(2,:) = MolSurad(2,:) + Dkonst_y.*nah(2,:);
    MolSurad(3,:) = MolSurad(3,:) + Dkonst_z.*nah(3,:);
    
    
    % korekcia castic za hranicou vymedzeneho priestoru
    Zx = MolSurad(1,:) > xh;
    MolSurad(1,Zx) = xh - (MolSurad(1,Zx)-xh);
    Zxx = MolSurad(1,:) < -xh;
    MolSurad(1,Zxx) = -xh - (MolSurad(1,Zxx)+xh);
    
    Zy = MolSurad(2,:) > yh;
    MolSurad(2,Zy) = yh - (MolSurad(2,Zy)-yh);
    Zyy = MolSurad(2,:) < -yh;
    MolSurad(2,Zyy) = -yh - (MolSurad(2,Zyy)+yh);
    
    Zz = MolSurad(3,:) > zh;
    MolSurad(3,Zz) = zh - (MolSurad(3,Zz)-zh);
    Zzz = MolSurad(3,:) < -zh;
    MolSurad(3,Zzz) = -zh - (MolSurad(3,Zzz)+zh);
    
    x = [-(xh+0.05*xh):0.00001:(xh+0.05*xh)];   %vektor pozdlz aplikovaneho gradientu
    vek_Nx = hist(MolSurad(1,:),x);             %castice su priradene do "kosov" podla stredov, ktore tvoria prvky vektoru x
    
    for j=1:Ng
        sig = vek_Nx.*exp(1i*2*pi*G(j)*x);      %vypocet signalu po aplikcii gradientov
        STG(t,j) = abs(sum(sig));
        BTG(t,j) = (2*pi*G(j))^2*ts(t);
    end
    
    
    %% vykreslovanie 
    figure(2)
    subplot(2,2,1)
    plot3(  [  xh xh xh xh xh -xh -xh -xh -xh -xh -xh xh xh -xh -xh xh],...
            [ -yh -yh yh yh -yh -yh -yh yh yh -yh -yh -yh yh yh yh yh],...
            [ -zh zh zh -zh -zh -zh zh zh -zh -zh  zh zh zh zh -zh -zh],'b');hold on;

    plot3(MolSurad(1,:),MolSurad(2,:),MolSurad(3,:),'r.');
       
    grid on
    axis equal
    xlabel('osa x') 
    ylabel('osa y')
    zlabel('osa z')
    title(sprintf('PO korekcii, cas= %g s',t*dt))
    hold off

    figure(2)
    subplot(2,2,2)
    hist(MolSurad(1,:),x);hold on
    title('histogram castic v ose x');
    hold off
    subplot(2,2,3)
    hist(MolSurad(2,:),x);hold on
    title('histogram castic v ose y')
    hold off
    subplot(2,2,4)
    hist(MolSurad(3,:),x);hold on
    title('histogram castic v ose z')
    hold off

end



figure()
plot(ts*1000,STG);
xlabel('diffusion time [ms]');
ylabel('signal');
title(sprintf('signal(diff\\_time) for diff\\_grad=m*%g rev/mm',g));
grid on;

figure()
plot(ts*1000,log10(STG));
xlabel('diffusion time [ms]');
ylabel('log10(signal)');
title(sprintf('signal(diff\\_time) for diff\\_grad=m*%g rev/mm)',g));
grid on;

figure()
plot(G,STG');
xlabel('diffusion-weighting gradient-moment [rev/mm]');
ylabel('signal');
title(sprintf('signal(diff\\_grad) for diff\\_time=%g..%g ms',dt,Ng*dt));
grid on;

figure()
plot(G,log10(STG'));
xlabel('diffusion-weighting gradient-moment [rev/mm]');
ylabel('log10(signal)');
title(sprintf('signal(diff\\_grad) for diff\\_time=%g..%g ms',dt,Ng*dt));
grid on;

figure()
plot(BTG(:),STG(:),'r.')
xlabel('b [s/mm^2]');
ylabel('signal');
grid on;

figure()
plot(BTG(:),log(STG(:)),'r.')
xlabel('b [s/mm^2]');
ylabel('ln(signal)');
grid on;